'use strict';

const fs = require('fs');
const https = require('https');
const retry = require('async-retry');
const httpStatusCodes = require('http-status-codes');
const jwt = require('jsonwebtoken');
const node_crypto = require('node:crypto');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  const n = Object.create(null);
  if (e) {
    for (const k in e) {
      n[k] = e[k];
    }
  }
  n.default = e;
  return n;
}

const https__namespace = /*#__PURE__*/_interopNamespaceCompat(https);
const retry__default = /*#__PURE__*/_interopDefaultCompat(retry);
const jwt__default = /*#__PURE__*/_interopDefaultCompat(jwt);

async function createHttpsAgent(pfxFilePath, passphraseFilePath) {
  if (!pfxFilePath || !passphraseFilePath) {
    throw new Error("Both pfxFilePath and passphraseFilePath must be provided");
  }
  const pfx = await fs.promises.readFile(pfxFilePath);
  const passphrase = await fs.promises.readFile(passphraseFilePath, "utf-8");
  return new https__namespace.Agent({
    pfx,
    passphrase: passphrase.trim(),
    rejectUnauthorized: false
  });
}

var __defProp$2 = Object.defineProperty;
var __defNormalProp$2 = (obj, key, value) => key in obj ? __defProp$2(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$2 = (obj, key, value) => {
  __defNormalProp$2(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class UnrecoverableServiceError extends Error {
  constructor(message, rootCause, status) {
    super(message);
    __publicField$2(this, "rootCause");
    __publicField$2(this, "status");
    this.name = this.constructor.name;
    this.rootCause = rootCause;
    this.status = status;
    Error.captureStackTrace(this, this.constructor);
  }
}

var __defProp$1 = Object.defineProperty;
var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$1 = (obj, key, value) => {
  __defNormalProp$1(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class RecoverableServiceError extends Error {
  constructor(message, rootCause, status) {
    super(message);
    __publicField$1(this, "rootCause");
    __publicField$1(this, "status");
    this.name = this.constructor.name;
    this.rootCause = rootCause;
    this.status = status;
    Error.captureStackTrace(this, this.constructor);
  }
}

var RecoverableBehaviour = /* @__PURE__ */ ((RecoverableBehaviour2) => {
  RecoverableBehaviour2["RECOVERABLE"] = "RECOVERABLE";
  RecoverableBehaviour2["UNRECOVERABLE"] = "UNRECOVERABLE";
  return RecoverableBehaviour2;
})(RecoverableBehaviour || {});
const hasHttpStatusConfigMatch = (receivedHttpStatus, errorBody, exceptHttpStatusConfig) => {
  if (!exceptHttpStatusConfig) {
    return false;
  }
  const allRelevantConfigs = exceptHttpStatusConfig.filter(
    (item) => item.httpStatus === receivedHttpStatus
  );
  if (allRelevantConfigs.length === 0) {
    return false;
  }
  return allRelevantConfigs.some((config) => {
    const { conditionFn } = config;
    if (!conditionFn) {
      return true;
    }
    return conditionFn(errorBody);
  });
};
const initHttpResponseHandler = (options, rootLogger) => {
  const log = rootLogger.child({ name: "http-client-response-handler" });
  const { defaultRecoverableBehaviour, exceptHttpStatusConfig, allowedStatusCodes } = options;
  return async (response, retryCount) => {
    if (response.ok) {
      return;
    }
    const { status } = response;
    if (allowedStatusCodes?.includes(status)) {
      return;
    }
    let errorBody;
    try {
      errorBody = await response.clone().json();
    } catch (e) {
      errorBody = await response.clone().text();
    }
    const message = `Error from ${response.url} with http status ${response.status}${retryCount != null ? ` on retry ${retryCount}` : ""}`;
    let unrecoverableServiceError;
    if (defaultRecoverableBehaviour === "UNRECOVERABLE" /* UNRECOVERABLE */) {
      unrecoverableServiceError = !hasHttpStatusConfigMatch(
        status,
        errorBody,
        exceptHttpStatusConfig
      );
    } else {
      unrecoverableServiceError = hasHttpStatusConfigMatch(
        status,
        errorBody,
        exceptHttpStatusConfig
      );
    }
    if (unrecoverableServiceError) {
      log.error(`${message} with errorBody: ${JSON.stringify(errorBody)}`);
      throw new UnrecoverableServiceError(message, errorBody, status);
    }
    log.warn(`${message} with errorBody: ${JSON.stringify(errorBody)}`);
    throw new RecoverableServiceError(message, errorBody, status);
  };
};

const initGraphQlHttpResponseHandler = (options, rootLogger) => {
  const log = rootLogger.child({
    name: "graphql-http-client-response-handler"
  });
  const handleErrorResponse = initHttpResponseHandler(options, log);
  const hasResponse = (networkError) => {
    return "response" in networkError;
  };
  return async (errorResponse) => {
    const { graphQLErrors, networkError } = errorResponse;
    if (networkError && hasResponse(networkError)) {
      const { response } = networkError;
      await handleErrorResponse(response);
    } else if (graphQLErrors) {
      const message = graphQLErrors.map((error) => {
        return error.message;
      }).join("\n");
      throw new UnrecoverableServiceError(message, null);
    }
  };
};

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class HttpRetryError extends Error {
  constructor(message, recoverable, errorOptions) {
    super(message, errorOptions);
    __publicField(this, "recoverable");
    this.name = this.constructor.name;
    this.recoverable = recoverable;
    Error.captureStackTrace(this, this.constructor);
  }
}

class ResponseValidationError extends Error {
}

const { sign } = jwt__default;
const FEIGN_HEADERS = {
  "cp-axa-user-id": "FEIGN",
  "cp-axa-user-role": "FEIGN"
};
const createXAxaContextHeaderV2 = ({
  clientId,
  audition,
  issuedAt
}) => {
  return sign(
    {
      iat: Math.round((issuedAt ?? /* @__PURE__ */ new Date()).getTime() / 1e3),
      iss: "https://api.axa.ch/x-axa-context/v2",
      client_id: clientId,
      jti: node_crypto.randomUUID(),
      aud: audition ?? "axahealth"
    },
    "secret",
    {
      header: {
        alg: "none",
        typ: void 0
      }
    }
  );
};

const fetchAndRetry = async ({ fetchFn, retryOptions, handleErrorResponseFn }) => {
  return retry__default(async (bail, retryCount) => {
    const response = await fetchFn();
    try {
      await handleErrorResponseFn(response, retryCount - 1);
    } catch (e) {
      const httpRetryError = new HttpRetryError(e.message, e instanceof RecoverableServiceError, {
        cause: e.cause
      });
      if (e instanceof UnrecoverableServiceError) {
        bail(httpRetryError);
      }
      throw httpRetryError;
    }
    return response;
  }, retryOptions);
};
const createDefaultErrorResponseHandler = (log) => initHttpResponseHandler(
  {
    defaultRecoverableBehaviour: RecoverableBehaviour.RECOVERABLE,
    exceptHttpStatusConfig: [
      { httpStatus: httpStatusCodes.StatusCodes.BAD_REQUEST },
      { httpStatus: httpStatusCodes.StatusCodes.FORBIDDEN }
    ],
    allowedStatusCodes: [httpStatusCodes.StatusCodes.NOT_FOUND]
  },
  log
);
const fetchJson = (input, init, userHeaders) => {
  return async () => fetch(input, {
    ...init,
    headers: {
      ...userHeaders,
      "Content-Type": "application/json",
      Accept: "application/json",
      ...init?.headers
    }
  });
};
const fetchJsonWithRetry = async ({
  fetch: fetch2,
  log,
  handleErrorResponseFn,
  validator,
  retryOptions = { retries: 3 }
}) => {
  const fetchFn = fetchJson(fetch2.input, fetch2.init, fetch2.user);
  const response = await fetchAndRetry({
    fetchFn,
    handleErrorResponseFn: handleErrorResponseFn ?? createDefaultErrorResponseHandler(log),
    retryOptions
  });
  const jsonResponse = await response.json();
  const jsonResponseParseResult = validator.safeParse(jsonResponse);
  if (!jsonResponseParseResult.success) {
    const logContext = { jsonResponse, err: jsonResponseParseResult.error };
    const fetchInput = ` ${JSON.stringify(fetch2.input)}`;
    const validationError = `Validation error while parsing response from ${fetchInput}. Error "${jsonResponseParseResult.error.message}" occurred.`;
    log.error(logContext, validationError);
    throw new ResponseValidationError(validationError);
  }
  return jsonResponseParseResult.data;
};
const fetchJsonFeignWithRetry = async (fetchJsonWithRetryRequest) => {
  return fetchJsonWithRetry({
    ...fetchJsonWithRetryRequest,
    fetch: { ...fetchJsonWithRetryRequest.fetch, user: FEIGN_HEADERS }
  });
};

exports.FEIGN_HEADERS = FEIGN_HEADERS;
exports.RecoverableBehaviour = RecoverableBehaviour;
exports.RecoverableServiceError = RecoverableServiceError;
exports.UnrecoverableServiceError = UnrecoverableServiceError;
exports.createDefaultErrorResponseHandler = createDefaultErrorResponseHandler;
exports.createHttpsAgent = createHttpsAgent;
exports.createXAxaContextHeaderV2 = createXAxaContextHeaderV2;
exports.fetchAndRetry = fetchAndRetry;
exports.fetchJson = fetchJson;
exports.fetchJsonFeignWithRetry = fetchJsonFeignWithRetry;
exports.fetchJsonWithRetry = fetchJsonWithRetry;
exports.initGraphQlHttpResponseHandler = initGraphQlHttpResponseHandler;
exports.initHttpResponseHandler = initHttpResponseHandler;
